"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.shorthands = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const logger_1 = __importDefault(require("@pnpm/logger"));
const render_help_1 = __importDefault(require("render-help"));
const setupOnWindows_1 = require("./setupOnWindows");
const errors_1 = require("./errors");
const rcOptionsTypes = () => ({});
exports.rcOptionsTypes = rcOptionsTypes;
const cliOptionsTypes = () => ({
    force: Boolean,
});
exports.cliOptionsTypes = cliOptionsTypes;
exports.shorthands = {};
exports.commandNames = ['setup'];
function help() {
    return (0, render_help_1.default)({
        description: 'Sets up pnpm',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'Override the PNPM_HOME env variable in case it already exists',
                        name: '--force',
                        shortAlias: '-f',
                    },
                ],
            },
        ],
        url: (0, cli_utils_1.docsUrl)('setup'),
        usages: ['pnpm setup'],
    });
}
exports.help = help;
function getExecPath() {
    if (process['pkg'] != null) {
        // If the pnpm CLI was bundled by vercel/pkg then we cannot use the js path for npm_execpath
        // because in that case the js is in a virtual filesystem inside the executor.
        // Instead, we use the path to the exe file.
        return process.execPath;
    }
    return (require.main != null) ? require.main.filename : process.cwd();
}
function copyCli(currentLocation, targetDir) {
    const newExecPath = path_1.default.join(targetDir, path_1.default.basename(currentLocation));
    if (path_1.default.relative(newExecPath, currentLocation) === '')
        return;
    logger_1.default.info({
        message: `Copying pnpm CLI from ${currentLocation} to ${newExecPath}`,
        prefix: process.cwd(),
    });
    fs_1.default.mkdirSync(targetDir, { recursive: true });
    fs_1.default.copyFileSync(currentLocation, newExecPath);
}
async function handler(opts) {
    var _a;
    const execPath = getExecPath();
    if (execPath.match(/\.[cm]?js$/) == null) {
        copyCli(execPath, opts.pnpmHomeDir);
    }
    const currentShell = detectCurrentShell();
    const updateOutput = await updateShell(currentShell, opts.pnpmHomeDir, { force: (_a = opts.force) !== null && _a !== void 0 ? _a : false });
    return `${updateOutput}

Setup complete. Open a new terminal to start using pnpm.`;
}
exports.handler = handler;
function detectCurrentShell() {
    if (process.env.ZSH_VERSION)
        return 'zsh';
    if (process.env.BASH_VERSION)
        return 'bash';
    if (process.env.FISH_VERSION)
        return 'fish';
    return typeof process.env.SHELL === 'string' ? path_1.default.basename(process.env.SHELL) : null;
}
async function updateShell(currentShell, pnpmHomeDir, opts) {
    switch (currentShell) {
        case 'bash':
        case 'zsh': {
            return reportShellChange(await setupShell(currentShell, pnpmHomeDir, opts));
        }
        case 'fish': {
            return reportShellChange(await setupFishShell(pnpmHomeDir, opts));
        }
    }
    if (process.platform === 'win32') {
        return (0, setupOnWindows_1.setupWindowsEnvironmentPath)(pnpmHomeDir, opts);
    }
    return 'Could not infer shell type.';
}
function reportShellChange({ action, configFile }) {
    switch (action) {
        case 'created': return `Created ${configFile}`;
        case 'added': return `Appended new lines to ${configFile}`;
        case 'updated': return `Replaced configuration in ${configFile}`;
        case 'skipped': return `Configuration already up-to-date in ${configFile}`;
    }
}
async function setupShell(shell, pnpmHomeDir, opts) {
    const configFile = path_1.default.join(os_1.default.homedir(), `.${shell}rc`);
    const content = `# pnpm
export PNPM_HOME="${pnpmHomeDir}"
export PATH="$PNPM_HOME:$PATH"
# pnpm end`;
    return {
        action: await updateShellConfig(configFile, content, opts),
        configFile,
    };
}
async function setupFishShell(pnpmHomeDir, opts) {
    const configFile = path_1.default.join(os_1.default.homedir(), '.config/fish/config.fish');
    const content = `# pnpm
set -gx PNPM_HOME "${pnpmHomeDir}"
set -gx PATH "$PNPM_HOME" $PATH
# pnpm end`;
    return {
        action: await updateShellConfig(configFile, content, opts),
        configFile,
    };
}
async function updateShellConfig(configFile, newContent, opts) {
    if (!fs_1.default.existsSync(configFile)) {
        await fs_1.default.promises.writeFile(configFile, newContent, 'utf8');
        return 'created';
    }
    const configContent = await fs_1.default.promises.readFile(configFile, 'utf8');
    const match = configContent.match(/# pnpm[\s\S]*# pnpm end/);
    if (!match) {
        await fs_1.default.promises.appendFile(configFile, `\n${newContent}`, 'utf8');
        return 'added';
    }
    if (match[0] !== newContent) {
        if (!opts.force) {
            throw new errors_1.BadShellSectionError({ current: match[1], wanted: newContent, configFile });
        }
        const newConfigContent = replaceSection(configContent, newContent);
        await fs_1.default.promises.writeFile(configFile, newConfigContent, 'utf8');
        return 'updated';
    }
    return 'skipped';
}
function replaceSection(originalContent, newSection) {
    return originalContent.replace(/# pnpm[\s\S]*# pnpm end/g, newSection);
}
//# sourceMappingURL=setup.js.map